#!/usr/bin/perl
use strict;

#
# $Id: RRDutil.pm 413 2006-07-08 05:15:52Z matt $
#

package RRDutil;

use CGI qw(:standard);
use Carp;
use POSIX qw(uname);
use lib ".";
use lib "lib";
use lib "/opt/local/lib/perl";
use RRDs;
use vars qw($VERSION);

$VERSION = '1.15';

=head1 NAME

RRDutil - Perl subroutines for rrdutil.pl


=head1 SYNOPSIS

RRDutil contains general subroutines related to data collection using rrdtool. 


=head1 DESCRIPTION


=cut

sub check_datadir;
sub check_html_dir;
sub create_rrd_db;
sub get_graph_types($$$);
sub get_hosts;
sub get_polls;
sub get_requested_graphs($@);
sub parse_config;

1;


sub new
{

=head1 new

To use these subroutines, you must first create a RRDutil object:

  use RRDutil;
  my $rrdutil = RRDutil->new();
  
once you have a rrdutil object, you may use any of the following methods. See the rrdutil.pl script for a working example.

=cut

	my ($class) = @_;
	my $self = { class => $class };
	bless ($self, $class);
	return $self;
};

sub get_graph_types($$$)
{

=head2 get_graph_types($$$)

Gets a list of graph types that are available for a supplied host. 

	my $graphs = $rrdutil->get_graph_types($host, $monitors, $conf);

$host is string containing a hostname.

$monitors is a list of graph types that are being monitored by rrdutil.

$conf is a hash containing all the key/values stored in rrdutil.conf.

The return is a hashref containing only the graph types available for the supplied host.
	
=cut

	my ($self, $host, $monitors, $conf) = @_;
	my (%hash, @list);

	my $monitortypes = $conf->{'mtypes'};
	
	my $polls = $monitors->{$host}; 

	foreach ( @$polls )
	{
		foreach my $type ( @$monitortypes ) 
		{
			$hash{$type} = 1 if $_->{$type};
		};
	};

	return \%hash;
};


sub get_requested_graphs($@)
{

=head2 get_requested_graphs($@)

When a HTTP user submits the form, it embeds a list of graph types they want to see. This function sorts that form input, tests to make sure the requested type is valid, and then returns a hashref containing the graphs type we are going to display in the HTTP form results.

	my $result = $rrdutil->get_requested_graphs($conf, @MonitorTypes);

$conf is a hashref containing the contents of the rrdutil.conf.

@MonitorTypes is a list of valid monitor types.

$result is a hashref containing the monitor types to display.

=cut


	my ($self, $conf, @MonitorTypes) = @_;

	my $debug = $conf->{'debug'};

	my %types;

	foreach my $type (@MonitorTypes)
	{
		if ( param($type) )
		{
			print "get_requested_graphs: type $type found.<br></br>\n" if $debug;
			$types{$type} = 1;
		}
	}

	if ( param('disk') ) { $types{'storage'} = 1; };

	return \%types;
};



=head2

If the RRD database doesn't exist, we need to create it. How it's created is dependent on what type of data will be stored in it. This subroutine contains all that logic.

	$rrdutil->create_rrd_db($data, $type, $mon, $description, $conf);

$data is the filename of the database.

$type is a valid rrdutil file type such as: net, disk, memory, storage, cpu, mail, etc...

$mon is a specific element to monitor, such as a "spam". Thus, $mon is a subset of $type.

$description is an english description of $mon.

$conf is a hashref with the contents of rrdutil.conf. 

=cut


sub create_rrd_db($$$$$)
{
	my ($self, $data, $type, $mon, $descr, $conf) = @_;
	my ($time, @rrdcmd);

	use RRDutil::SNMP;

	my $debug = $conf->{'debug'};

	$time = time unless ( $time );

	push @rrdcmd, $data;
	push @rrdcmd, "--step=300";
	push @rrdcmd, "--start=$time";

	if ($type eq "net") 
	{
		push @rrdcmd, "DS:in:COUNTER:600:U:U";
		push @rrdcmd, "DS:out:COUNTER:600:U:U";
	} 
	elsif ($type eq "disk") 
	{
		push @rrdcmd, "DS:reads:COUNTER:600:U:U";
		push @rrdcmd, "DS:writes:COUNTER:600:U:U";
		push @rrdcmd, "DS:read:COUNTER:600:U:U";
		push @rrdcmd, "DS:written:COUNTER:600:U:U";
	} 
	elsif ($type eq "memory") 
	{
		push @rrdcmd, "DS:swap:GAUGE:600:0:4000000";
		push @rrdcmd, "DS:swapavail:GAUGE:600:0:9000000";
		push @rrdcmd, "DS:total:GAUGE:600:0:9000000";
		push @rrdcmd, "DS:avail:GAUGE:600:0:4000000";
		push @rrdcmd, "DS:free:GAUGE:600:0:9000000";
		push @rrdcmd, "DS:shared:GAUGE:600:0:4000000";
		push @rrdcmd, "DS:buffer:GAUGE:600:0:1000000";
		push @rrdcmd, "DS:cached:GAUGE:600:0:1000000";
	}
	elsif ($type eq "storage")
	{
		push @rrdcmd, "DS:total:GAUGE:600:U:U";
		push @rrdcmd, "DS:avail:GAUGE:600:U:U";
		push @rrdcmd, "DS:used:GAUGE:600:U:U";
		push @rrdcmd, "DS:percent:GAUGE:600:U:U";
	}
	elsif ($type eq "load")
	{
		push @rrdcmd, "DS:one:GAUGE:600:U:U";
		push @rrdcmd, "DS:five:GAUGE:600:U:U";
		push @rrdcmd, "DS:fifteen:GAUGE:600:U:U";
		push @rrdcmd, "DS:user:GAUGE:600:U:U";
		push @rrdcmd, "DS:system:GAUGE:600:U:U";
	}
	elsif ($type eq "cpu")
	{
		push @rrdcmd, "DS:user:GAUGE:600:U:U";
		push @rrdcmd, "DS:system:GAUGE:600:U:U";
		push @rrdcmd, "DS:idle:GAUGE:600:U:U";
	}
	elsif ($type eq "www")
	{
		push @rrdcmd, "DS:hits:GAUGE:600:U:U";
	}
	elsif ($type eq "mail")
	{
		if ($mon eq "send")
		{
			push @rrdcmd, "DS:delivery_success:COUNTER:600:0:U";
			push @rrdcmd, "DS:delivery_deferral:COUNTER:600:0:U";
			push @rrdcmd, "DS:delivery_failure:COUNTER:600:0:U";
			push @rrdcmd, "DS:message_bytes:COUNTER:600:0:U";
			push @rrdcmd, "DS:message_new:COUNTER:600:0:U";
			push @rrdcmd, "DS:message_bounce:COUNTER:600:0:U";
			push @rrdcmd, "DS:message_end:COUNTER:600:0:U";
			push @rrdcmd, "DS:concurrencyremote:GAUGE:600:0:100";
		} 
		elsif ( $mon eq "mua" )
		{
			push @rrdcmd, "DS:pop3_connect:COUNTER:600:0:U";
			push @rrdcmd, "DS:pop3_ssl_connect:COUNTER:600:0:U";
			push @rrdcmd, "DS:pop3_success:COUNTER:600:0:U";
			push @rrdcmd, "DS:pop3_ssl_success:COUNTER:600:0:U";
			push @rrdcmd, "DS:imap_connect:COUNTER:600:0:U";
			push @rrdcmd, "DS:imap_ssl_connect:COUNTER:600:0:U";
			push @rrdcmd, "DS:imap_success:COUNTER:600:0:U";
			push @rrdcmd, "DS:imap_ssl_success:COUNTER:600:0:U";
			push @rrdcmd, "DS:webmail_connect:COUNTER:600:0:U";
			push @rrdcmd, "DS:webmail_success:COUNTER:600:0:U";
			push @rrdcmd, "DS:smtpauth_connect:COUNTER:600:0:U";
			push @rrdcmd, "DS:smtpauth_success:COUNTER:600:0:U";
		}
		elsif ( $mon eq "rbl" || $mon eq "smtp")
		{
			push @rrdcmd, "DS:smtp1:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp2:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp3:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp4:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp5:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp6:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp7:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp8:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp9:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp10:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp11:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp12:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp13:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp14:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp15:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp16:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp17:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp18:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp19:COUNTER:600:0:100000";
			push @rrdcmd, "DS:smtp20:COUNTER:600:0:100000";
		}
		elsif ( $mon eq "spam" )
		{
			push @rrdcmd, "DS:sa1:COUNTER:600:0:U";
			push @rrdcmd, "DS:sa2:COUNTER:600:0:U";
			push @rrdcmd, "DS:sa3:GAUGE:600:0:U";
			push @rrdcmd, "DS:sa4:GAUGE:600:0:U";
			push @rrdcmd, "DS:sa5:GAUGE:600:0:U";
			push @rrdcmd, "DS:sa6:GAUGE:600:0:U";
			push @rrdcmd, "DS:sa7:GAUGE:600:0:U";
			push @rrdcmd, "DS:sa8:GAUGE:600:0:U";
			push @rrdcmd, "DS:sa9:GAUGE:600:0:U";
			push @rrdcmd, "DS:sa10:GAUGE:600:0:U";
		};
	}
	elsif ($type eq "mysql")
	{
		if ( $mon eq "queries" )
		{
			push @rrdcmd, "DS:mysql_queriessec:COUNTER:600:0:5000";
			push @rrdcmd, "DS:mysql_connections:COUNTER:600:0:5000";
			push @rrdcmd, "DS:mysql_slowqueries:COUNTER:600:0:5000";
		}
		elsif ( $mon eq "type_queries" )
		{
			push @rrdcmd, "DS:mysql_select:COUNTER:600:0:5000";
			push @rrdcmd, "DS:mysql_changedb:COUNTER:600:0:5000";
			push @rrdcmd, "DS:mysql_update:COUNTER:600:0:5000";
			push @rrdcmd, "DS:mysql_insert:COUNTER:600:0:5000";
			push @rrdcmd, "DS:mysql_delete:COUNTER:600:0:5000";
		}
		elsif ( $mon eq "combined" )
		{
			push @rrdcmd, "DS:my1:COUNTER:600:0:5000";
			push @rrdcmd, "DS:my2:COUNTER:600:0:5000";
			push @rrdcmd, "DS:my3:COUNTER:600:0:5000";
			push @rrdcmd, "DS:my4:COUNTER:600:0:5000";
			push @rrdcmd, "DS:my5:COUNTER:600:0:5000";
			push @rrdcmd, "DS:my6:COUNTER:600:0:5000";
			push @rrdcmd, "DS:my7:COUNTER:600:0:5000";
			push @rrdcmd, "DS:my8:COUNTER:600:0:5000";
		}
	}
	elsif ($type eq "mbmon")
	{
		if ( $mon eq "temperature" )
		{
			push @rrdcmd, "DS:mbmon_temp0:GAUGE:600:-273:200";
			push @rrdcmd, "DS:mbmon_temp1:GAUGE:600:-273:200";
			push @rrdcmd, "DS:mbmon_temp2:GAUGE:600:-273:200";
		}
		elsif ( $mon eq "voltage" )
		{
			push @rrdcmd, "DS:mbmon_vc0:GAUGE:600:0:260";
			push @rrdcmd, "DS:mbmon_vc1:GAUGE:600:0:260";
			push @rrdcmd, "DS:mbmon_v33:GAUGE:600:0:260";
			push @rrdcmd, "DS:mbmon_v50p:GAUGE:600:0:260";
			push @rrdcmd, "DS:mbmon_v12p:GAUGE:600:0:260";
			push @rrdcmd, "DS:mbmon_v12n:GAUGE:600:0:260";
			push @rrdcmd, "DS:mbmon_v50n:GAUGE:600:0:260";
		}
		elsif ( $mon eq "fanspeed" )
		{
			push @rrdcmd, "DS:mbmon_fan0:GAUGE:600:0:U";
			push @rrdcmd, "DS:mbmon_fan1:GAUGE:600:0:U";
			push @rrdcmd, "DS:mbmon_fan2:GAUGE:600:0:U";
		}
	}
	elsif ( $type eq "apache" )
	{
		if ( $mon eq "workers" )
		{
			push @rrdcmd, "DS:apache_workers_busy:GAUGE:600:0:U";
			push @rrdcmd, "DS:apache_workers_idle:GAUGE:600:0:U";
		}
		elsif( $mon eq "bytes_per_req" )
		{
			push @rrdcmd, "DS:apache_per_req:GAUGE:600:0:U";
		}
		elsif( $mon eq "req_per_sec" )
		{
			push @rrdcmd, "DS:apache_req_per_sec:GAUGE:600:0:U";
		}
		elsif( $mon eq "combined" )
		{
			push @rrdcmd, "DS:workers_busy:GAUGE:600:0:U";
			push @rrdcmd, "DS:workers_idle:GAUGE:600:0:U";
			push @rrdcmd, "DS:bytes_per_req:GAUGE:600:0:U";
			push @rrdcmd, "DS:req_per_sec:GAUGE:600:0:U";
		}
	}
	elsif ( $type eq "pf" )
	{
		if ( $mon eq "states" )
		{
 			push @rrdcmd, "DS:states_entries:ABSOLUTE:600:0:U";
 			push @rrdcmd, "DS:states_searches:DERIVE:600:0:U";
 			push @rrdcmd, "DS:states_inserts:DERIVE:600:0:U";
 			push @rrdcmd, "DS:states_removals:DERIVE:600:0:U";
		}
		elsif ( $mon eq "ipbytes" )
		{
            push @rrdcmd, "DS:bytes_v4_in:DERIVE:600:0:U";          
            push @rrdcmd, "DS:bytes_v4_out:DERIVE:600:0:U";
            push @rrdcmd, "DS:bytes_v6_in:DERIVE:600:0:U";           
            push @rrdcmd, "DS:bytes_v6_out:DERIVE:600:0:U";
        }
		elsif ( $mon eq "firewall" )
		{
			push @rrdcmd, "DS:passed:DERIVE:600:0:U";
			push @rrdcmd, "DS:blocked:DERIVE:600:0:U";
		}
	
	}
	elsif ($type eq "dns")
	{
	}
	else 
	{
		carp "ERROR: polls of type $type are not supported (yet)!\n";
		return 0;
	};

	if ($type eq "www")
	{
		push @rrdcmd, "RRA:AVERAGE:0.5:1:600";
		push @rrdcmd, "RRA:AVERAGE:0.5:12:775";
		push @rrdcmd, "RRA:AVERAGE:0.5:288:797";
	} 
	else 
	{
		push @rrdcmd, "RRA:AVERAGE:0.5:1:600";
		push @rrdcmd, "RRA:AVERAGE:0.5:6:700";
		push @rrdcmd, "RRA:AVERAGE:0.5:24:775";
		push @rrdcmd, "RRA:AVERAGE:0.5:288:797";

		push @rrdcmd, "RRA:MAX:0.5:1:600";
		push @rrdcmd, "RRA:MAX:0.5:6:700";
		push @rrdcmd, "RRA:MAX:0.5:24:775";
		push @rrdcmd, "RRA:MAX:0.5:288:797";

		push @rrdcmd, "RRA:MIN:0.5:1:600";
		push @rrdcmd, "RRA:MIN:0.5:6:700";
		push @rrdcmd, "RRA:MIN:0.5:24:775";
		push @rrdcmd, "RRA:MIN:0.5:288:797";
	};

	print "create_rrd_db: creating $data...";

	RRDs::create ( @rrdcmd );

	return RRDutil::SNMP::check_rrds_error($debug);

	# --start   time in seconds since epoch for first value
	
	# DS        data source (DS:ds-name:DST:heartbeat:min:max
	# ds-name   1 to 19 characters [a-zA-Z0-0_]
	# DST       GAUGE, COUNTER, DERIVE, ABSOLUTE
	# heartbeat max seconds between updates before assumed Unknown
	# min/max   optional parameters for valid data range 
	
	# RRA       RRArchive (RRA:CF:xff:steps:rows)
	# CF        Consolidate function (AVERAGE, MIN, MAX, LAST)
	# xff       xfiles factor - what % of data may be unknown
	# steps     how many primary data points to build a consolidated point
	# rows      how many values are kept

	# RRA:::1:600   - two days of 5 min data
	# RRA:::6:700   - two weeks of 30 min
	# RRA:::24:775  - two months of 2 hour
	# RRA:::288:797 - 797 days of 1 day

	# to measure:
	#  GAUGE    - temperature, cpu, load
	#  COUNTER  - network, (any rate/time)
	#  DERIVE   -
	#  ABSOLUTE -
} 

sub parse_config($$;$)
{

=head2 parse_config

parses the rrdutil.conf file and returns the contents in a hashref

	my $r = $rrdutil->parse_config($hostlist, $conf, $debug);

=cut

	my ($self, $hostlist, $conf, $debug) = @_;
	my (%confme, %hash);

	# extract the key value pairs from $conf into %confme
	#
	# the reason for this is long and convoluted but we want to
	# use $conf values later and because it's a hashref, the only
	# way I can safely use delete on it now and still get to those
	# values is by moving the values from the ref to a private hash.

	foreach my $key ( keys %$conf ) { $confme{$key} = $conf->{$key}; };

	foreach my $host ( @$hostlist )
	{
		print "parse_config:  host $host\n" if $debug;
		my @polls;
	
		KEY: foreach my $key ( keys %confme )
		{
			unless ( $confme{$key} =~ /^.*,.*,.*,/ )
			{
				delete $confme{$key};
				next KEY;
			};

			if ( $confme{$key} =~ /\s+$/ )
			{
				print "found a trailing space in $confme{$key}\n" if $debug;
				($confme{$key}) = $confme{$key} =~ /^(.*)\s+$/;
			};

			$confme{$key} =~ /^\s*(.*),\s*(.*),\s*(.*),\s*(.*)\s*(.*)/;

			unless ( $1 eq $host )
			{
				# delete $confme{$key};
				next KEY;
			};

			if ( $2)
			{
				my %poll = ( $2 => [ $3, $4 ] );  push @polls, \%poll;
			};
			
			delete $confme{$key};
		};
		
		$hash{$host} = \@polls;
	};

	return \%hash;
};

sub get_hosts($;$)
{

=head2 get_hosts

  my $hostlist   = $rrdutil->get_hosts($conf);

Pass the contents of rrdutil.conf in $conf and it'll parse and return a list of hosts

=cut

	my ($self, $confme, $debug) = @_;
	my (@hosts, %host);

	foreach my $key ( keys %$confme )
	{
		if ( $confme->{$key} =~ /snmp/ )
		{
			$confme->{$key} =~ /\s*(.*),\s*(.*),\s*(.*),\s*(.*)/;
			$host{$1} = 1;
		};
	};

	foreach my $key ( keys %host )
	{
		print "get_hosts: hosts: $key \n" if $debug;
#		print " hosts: $key \n" if $debug;
		push @hosts, $key;
	};

	return \@hosts;
};

sub get_polls($$$)
{

=head2 get_polls

	foreach my $poll ( $rrdutil->get_polls($host, "load", $monitors) )
	{
		print "<IMG SRC=$url/$host/${poll}.png><br></br>";
		print "<IMG SRC=$url/$host/${poll}_detail.png><br></br>";
	};

=cut


	my ($self, $host, $type_selected, $monitors) = @_;
	my @polllist;

	my $polls = $monitors->{$host};

	POLL: foreach my $poll (@$polls)
	{
		my ($type, $detail);
		until ( $type ) { ($type, $detail) = each %$poll };		

		next POLL unless ( $type eq $type_selected );
		
		my $mon = $detail->[0];
		my $descr = $detail->[1];

		if ($type eq "www") 
		{
			if ($mon eq "solo" ) 
			{
				push @polllist, $descr;
			} 
			else 
			{
				push @polllist, "webhits";
			};
		} 
		else 
		{
			push @polllist, $descr;
		};
	};
	
	return sort @polllist;
};


sub check_datadir($$$)
{

=head2 check_datadir

Make sure the permissions on the rrdutil data directory are ok. Return a useful error message if not.

	$rrdutil->check_datadir($host, $perm, $conf);

=cut

	my ($self, $host, $perm, $conf) = @_;

	my $datadir = $conf->{'datadir'};

	unless ( -d "$datadir/$host" ) 
	{
		unless ( mkdir("$datadir/$host", 0755) )
		{
			carp "WARN: couldn't create $datadir/$host: $!\n";
			return 0;
		};
	};

	if ( $perm eq "write" && !-w "$datadir/$host") 
	{
		carp "WARN: I need write access to $datadir/$host.\n";
		return 0;
	};

	if ( $perm eq "read" && ! -r "$datadir/$host") 
	{
		carp "WARN: I need write access to $datadir/$host.\n";
		return 0;
	};

	return 1;
}

sub check_html_dir($$$)
{

=head2 check_html_dir

=cut


	my ($self, $htmldir, $host, $perm) = @_;

	unless ( -d "$htmldir/$host" ) 
	{
		unless ( mkdir("$htmldir/$host", 0755) )
		{
			carp "WARN: couldn't create $htmldir/$host: $!\n";
			return 0;
		};
	};

	if ( $perm eq "write" && !-w "$htmldir/$host") 
	{
		carp "WARN: I need write access to $htmldir/$host.\n";
		return 0;
	};

	if ( $perm eq "read" && ! -r "$htmldir/$host") 
	{
		carp "WARN: I need write access to $htmldir/$host.\n";
		return 0;
	};

	return 1;
}


1;
__END__


COPYRIGHT

Copyright (c) 2003-2004, The Network People, Inc. All Rights Reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

Neither the name of the The Network People, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

