#!/usr/bin/perl
use strict;

#
# $Id: SNMP.pm 413 2006-07-08 05:15:52Z matt $
#

package RRDutil::SNMP;

use Carp;
use POSIX qw(uname);
use lib ".";
use lib "../";
use RRDutil; my $rrdutil = RRDutil->new();

use vars qw($VERSION);

$VERSION = '1.4';

=head1 NAME

RRDutil::SNMP - Perl module for rrdutil.pl


=head1 SYNOPSIS

RRDutil::SNMP


=head1 DESCRIPTION

=cut

sub new
{
	my ($class, $name) = @_;
	my $self = { name => $name };
	bless ($self, $class);
	return $self;
};

sub update_rrds($$$)
{
	my ($self, $hosts, $monitors, $conf) = @_;

	my $debug    = $conf->{'debug'};
	my $Monitors = $conf->{'monitorhash'};
	my $datadir  = $conf->{'datadir'};

	foreach my $host ( @$hosts )
	{
		print "update_rrds: hostname: $host \n" if $debug;

		my $polls = $monitors->{$host}; 

		my $snmp = undef;
		
		foreach my $poll ( @$polls )    # iterate through @polls looking for SNMP
		{
			next unless ( $poll->{'snmp'} && $poll->{'snmp'}->[0] ne 0 );
			$snmp = $poll->{'snmp'};
		};

		next unless ($snmp);

		my $session = start_session( $conf, $host, $snmp);
		next unless ($session);

		POLL: foreach my $poll (@$polls) 
		{
			next unless ( $rrdutil->check_datadir( $host, "write", $conf ) );

			my ($type, $detail);
			until ( $type ) { ($type, $detail) = each %$poll };

			my $mon = $detail->[0];
			my $descr = $detail->[1];

			print "update_rrds host:  $host   type: $type   mon: $mon   descr: $descr\n" if $debug;

			next POLL unless ( $Monitors->{$type} && $Monitors->{$type} == 1 );
				
			my $file  = "$datadir/$host/$descr.rrd";

   			return 0 unless ( RRDutil::SNMP::CheckRRD($file, $type, $mon, $descr, $conf) );

			if    ($type eq "net")     { UpdateNetworkSNMP  ($host, $session, $type, $mon, $descr, $conf) }
			elsif ($type eq "disk")    { UpdateDiskActivitySNMP($file, $session, $mon, $descr, $conf)     }
			elsif ($type eq "memory")  { UpdateMemoryUseSNMP($file, $session, $conf)                      }
			elsif ($type eq "storage") { UpdateStorageSNMP  ($host, $session, $type, $mon, $descr, $conf) }
			elsif ($type eq "load")    { UpdateLoadAvgSNMP  ($file, $session, $conf)                      }
			elsif ($type eq "cpu")     { UpdateCPUStateSNMP ($file, $session, $conf)                      } 
			elsif ($type eq "info")    { UpdateSystemSNMP   ($file, $session, $conf)                      }
			elsif ($type eq "www")     { UpdateWebHitsSNMP  ($file, $session, $type, $mon, $descr, $conf) } 
			elsif ($type eq "mail")    { UpdateMailSNMP     ($file, $session, $type, $mon, $descr, $conf) } 
			elsif ($type eq "mysql")   { UpdateMysqlSNMP    ($file, $session, $type, $mon, $descr, $conf) } 
			elsif ($type eq "mbmon")   { UpdateMbmonSNMP    ($file, $session, $type, $mon, $descr, $conf) }
			elsif ($type eq "apache")  { UpdateApacheSNMP   ($file, $session, $type, $mon, $descr, $conf) }
			elsif ($type eq "pf")      { UpdatePFSNMP       ($file, $session, $type, $mon, $descr, $conf) }
			else 
			{
				print "UpdateRRD: skipping type $type\n";
				return 0;
			};
		};

		$session->close;
	};
};


sub CheckRRD($$$$$)
{
	my ($file, $type, $mon, $descr, $conf) = @_;

	unless ( -e $file)
	{
		unless ( $rrdutil->create_rrd_db($file, $type, $mon, $descr, $conf) )
		{
			print "WARNING: create_rrd_db $file failed.\n";
			return 0;
		};
	};

	use RRDutil::Utility; my $utility = RRDutil::Utility->new();
	return ( $utility->file_check_writable($file) );
};

sub CramKeyValuePairsIntoHash($)
{
	my ($r) = @_;
	my (%hash, $done);

	foreach my $key ( keys %$r ) 
	{
		my $list = $r->{$key};
		my @pairs = split(":", $list);
		$done = 0;
		do
		{
			my $a = shift @pairs;
			my $b = shift @pairs;

			if ($a) { $hash{$a} = $b; } 
			else    { $done = 1;      };
		} 
		until ($done);
	};
	return \%hash;
};


##
# SNMP collections
##
# -----------------------------------------------------------------------------

sub check_rrds_error(;$)
{
	my ($debug) = @_;

	use RRDs;

	if (RRDs::error) 
	{ 
		_formatted(undef, "RRDs ERROR: ". RRDs::error, "FAILED");
		return 0;
	} 
	else 
	{ 
		print "success.\n" if $debug; 
		return 1;
	};
}


sub start_session($$$;$)
{
	my ($conf, $host, $vals) = @_;

	my $timeout = $conf->{'timeout'} || 8;
	   $timeout = $timeout * 1;   # make sure it's numeric
	my $debug   = $conf->{'debug'}   || 0;

	my ($session, $error);

	# try p5-SNMP
	my $module = "SNMP";

#	print "start_session: checking for SNMP..." if $debug;
#	if ( eval "require $module" ) {
#		$module->import();
#		print "yes\n" if $debug;
#		$session  = new SNMP::Session(
#			Community=> $vals->[1],
#			DestHost => $host, 
#			Timeout  => $timeout,
#			Version  => 2,
#		);
#	}
#	print "nope.\n" if ( $debug&& ! defined $session);;

	unless (defined($session)) {
		print "start_session: trying Net::SNMP..." if $debug;
		$module = "Net::SNMP";

		unless ( eval "require $module" ) {
			require RRDutil::Perl;
			my $perl = new RRDutil::Perl;
			if ( -d "/usr/ports/net-mgmt" ) {
				$perl->module_load( {module=>"Net::SNMP", ports_name=>"p5-Net-SNMP", ports_group=>"net-mgmt"} );
			} else {
				$perl->module_load( {module=>"Net::SNMP", ports_name=>"p5-Net-SNMP", ports_group=>"net"} );
			};
		};

		if ( eval "require $module" ) {
			print "ok.\n" if $debug;
			$module->import();

			($session, $error) = Net::SNMP->session(
				-hostname  => $host,
				-port      => 161,
				-version   => $vals->[0],
				-community => $vals->[1],
				-timeout   => $timeout
			);
		};
	} 

	unless (defined($session)) 
	{
		print "start_session: hostname: $host\n";
		print "start_session: version:  $vals->[0]\n";
		print "start_session: communit: $vals->[1]\n";
		printf("start_session: ERROR: %s.\n", $error);
		return 0;
	} 

	return $session;
};

sub UpdateCPUStateSNMP($$$)
{
	my ($file, $session, $conf) = @_;

	my $userOid   = $conf->{'cpu_userOid'}    ||  ".1.3.6.1.4.1.2021.11.9.0";
	my $systemOid = $conf->{'cpu_systemOid'}  || ".1.3.6.1.4.1.2021.11.10.0";
	my $idleOid   = $conf->{'cpu_idleOid'}    || ".1.3.6.1.4.1.2021.11.11.0";

	my $debug     = $conf->{'debug'};

	my $r = $session->get_request(
		-varbindlist => [$userOid, $systemOid, $idleOid] 
	);

	unless (defined($r)) { 
		printf("NetIfSNMP: ERROR: %s.\n", $session->error); 
		return 0;
	}

	print "UpdateCPUStateSNMP: user: $r->{'user'}  system: $r->{'system'}\n" if $debug;

	RRDs::update ( $file, "--template=user:system:idle", 
		"N:$r->{$userOid}:$r->{$systemOid}:$r->{$idleOid}"  );
	return( check_rrds_error() );
};

sub UpdateLoadAvgSNMP($$$)
{
	my ($file, $session, $conf) = @_;

	my $oneOid     = $conf->{'load_oneOid'};
	my $fiveOid    = $conf->{'load_fiveOid'};
	my $fifteenOid = $conf->{'load_fifteenOid'};
	my $userOid    = $conf->{'cpu_userOid'};
	my $systemOid  = $conf->{'cpu_systemOid'};
	my $debug      = $conf->{'debug'};

	my $r = $session->get_request(
		-varbindlist => [$oneOid, $fiveOid, $fifteenOid, $userOid, $systemOid] 
	);

	unless ( defined($r) )
	{ 
		_formatted(undef, "UpdateLoadAvgSNMP: polling load averages", "FAILED") if $debug;
		printf("\tSNMP ERROR: %s.\n", $session->error); 
		return 0;
	} 

	_formatted(undef, "UpdateLoadAvgSNMP: polling load averages", "ok") if $debug;

	my $template = "--template=one:five:fifteen:user:system";
	my $values   = "N:$r->{$oneOid}:$r->{$fiveOid}:$r->{$fifteenOid}:$r->{$userOid}:$r->{$systemOid}";

	if ( $r->{$userOid} =~ /noSuchObject/ ) {
		$template = "--template=one:five:fifteen";
		$values = "N:$r->{$oneOid}:$r->{$fiveOid}:$r->{$fifteenOid}";
	};

	print "\trrdtool update $template\n\t$values\n" if $debug;

	RRDs::update ($file, $template, $values);

	return( check_rrds_error() );
};

sub UpdateMemoryUseSNMP($$$)
{
	my ($file, $session, $conf) = @_;

	my $debug     = $conf->{'debug'};

	my $swapOid       = $conf->{'mem_swapOid'}     || ".1.3.6.1.4.1.2021.4.3.0";
	my $swapAvailOid  = $conf->{'mem_swapAvailOid'} || ".1.3.6.1.4.1.2021.4.4.0";
	my $totalOid      = $conf->{'mem_totalOid'}    || ".1.3.6.1.4.1.2021.4.5.0";
	my $availOid      = $conf->{'mem_availOid'}    || ".1.3.6.1.4.1.2021.4.6.0";
	my $freeOid       = $conf->{'mem_freeOid'}     || ".1.3.6.1.4.1.2021.4.11.0";
	my $sharedOid     = $conf->{'mem_sharedOid'}   || ".1.3.6.1.4.1.2021.4.13.0";
	my $bufferOid     = $conf->{'mem_bufferOid'}   || ".1.3.6.1.4.1.2021.4.14.0";
	my $cachedOid     = $conf->{'mem_cachedOid'}   || ".1.3.6.1.4.1.2021.4.15.0";

	my $r = $session->get_request( 
		-varbindlist => [ $swapOid, $swapAvailOid, $totalOid, 
			$availOid, $freeOid, $sharedOid, $bufferOid, $cachedOid ] 
	);

	unless ( defined($r) )
	{ 
		printf("GetMemorySNMP: SNMP ERROR: %s.\n", $session->error); 
		return 0;
	} 

	_formatted(undef, "GetMemorySNMP: polled memory info", "ok") if $debug;

	my $template = "--template=swap:swapavail:total:avail:free:shared:buffer:cached";
	my $values   = "N:$r->{$swapOid}:$r->{$swapAvailOid}:$r->{$totalOid}:$r->{$availOid}:$r->{$freeOid}:$r->{$sharedOid}:$r->{$bufferOid}:$r->{$cachedOid}";

	print "\trrdtool update $file\n\t$template\n\t$values\n" if $debug;

	RRDs::update ( $file, $template, $values );

	return( check_rrds_error() );
};

sub UpdateStorageSNMP($$$$$$)
{
	my ($host, $session, $type, $mon, $descr, $conf) = @_;
	my ($num, $r, $pathOid);

	my $datadir = $conf->{'datadir'};
	my $debug   = $conf->{'debug'};

	my $file   = "$datadir/$host/$descr.rrd";
	return 0 unless CheckRRD($file, $type, $mon, $descr, $conf);

	my $i = 1;
	do
	{
		$pathOid = "$conf->{'disk_pathOid'}$i" || ".1.3.6.1.4.1.2021.9.1.2.$i";
		$r = $session->get_request( -varbindlist=>[ $pathOid ] );
	
		if ($r->{$pathOid} eq $mon) {
			_formatted(undef, "UpdateStorageSNMP: polling for $mon", "ok") if $debug;
			$num = $i;
		};
		$i++;
	} 
	until ( $r->{$pathOid} eq $mon || $i == 10 || $r->{$pathOid} =~ /No Such/ );

	unless ($num) 
	{
		_formatted(undef, "UpdateStorageSNMP: polling for $mon", "FAILED");
		return 0;
	};

	my $totalOid   = "$conf->{'disk_totalOid'}$num"   || ".1.3.6.1.4.1.2021.9.1.6.$num";
	my $availOid   = "$conf->{'disk_availOid'}$num"   || ".1.3.6.1.4.1.2021.9.1.7.$num";
	my $usedOid    = "$conf->{'disk_usedOid'}$num"    || ".1.3.6.1.4.1.2021.9.1.8.$num";
	my $percentOid = "$conf->{'disk_percentOid'}$num" || ".1.3.6.1.4.1.2021.9.1.9.$num";

	$num = 0;

	$r = $session->get_request( 
		-varbindlist => [ $totalOid, $availOid, $usedOid, $percentOid ] 
	);

	unless ( defined($r) )
	{
		_formatted(undef, "        polling details for $mon", "FAILED");
		printf("UpdateStorageSNMP: ERROR: %s.\n", $session->error); 
		return 0;
	} 

	_formatted(undef, "        polling details for $mon", "ok") if $debug;

	my $template = "--template=percent:total:avail:used";
	my $values   = "N:$r->{$percentOid}:$r->{$totalOid}:$r->{$availOid}:$r->{$usedOid}";

	print "\trrdtool update $file\n\t$template $values\n" if $debug;

	RRDs::update ( $file, $template, $values);

	return( check_rrds_error() );
};

sub UpdateDiskActivitySNMP($$$$$)
{
	my ($file, $session, $mon, $descr, $conf) = @_;
	my ($num, $r, $diskListOid);

	my $debug   = $conf->{'debug'};

	my $i = 1;
	do
	{
		$diskListOid = "$conf->{'disk_diskListOid'}$i";
		# $diskListOid = ".1.3.6.1.4.1.2021.13.15.1.1.2.$i";

		$r = $session->get_request( -varbindlist=>[ $diskListOid ] );
	
		if ($r->{$diskListOid} eq $mon)
		{
			_formatted(undef, "UpdateDiskActivitySNMP: searching for disk $mon", "disk $i") if $debug;
			$num = $i;
		};
		$i++;
	} 
	until ( $r->{$diskListOid} eq $mon || $i == 10 || $r->{$diskListOid} =~ /No Such/ );

	unless ($num) {
		_formatted(undef, "UpdateDiskActivitySNMP: searching for disk $mon", "FAILED");
		return 0;
	};

	my $readsOid    = "$conf->{'disk_readsOid'}$num"   || ".1.3.6.1.4.1.2021.13.15.1.1.5.$num";
	my $writesOid   = "$conf->{'disk_writesOid'}$num"  || ".1.3.6.1.4.1.2021.13.15.1.1.6.$num";
	my $readOid     = "$conf->{'disk_readOid'}$num"    || ".1.3.6.1.4.1.2021.13.15.1.1.3.$num";
	my $writtenOid  = "$conf->{'disk_writtenOid'}$num" || ".1.3.6.1.4.1.2021.13.15.1.1.4.$num";
	my $descOid     = "$conf->{'disk_descOid'}$num"    || ".1.3.6.1.4.1.2021.13.15.1.1.2.$num";

	$r = $session->get_request( 
		-varbindlist => [ $descOid, $readsOid, $writesOid, $readOid, $writtenOid ] 
	);

	unless ( defined($r) )
	{
		_formatted(undef, "\tSNMP polling ERROR: " .$session->error, "FAILED"); 
		return 0;
	} 

	$r->{'descr'}   = $r->{$descOid};
	$r->{'reads'}   = $r->{$readsOid};
	$r->{'writes'}  = $r->{$writesOid};
	$r->{'read'}    = $r->{$readOid};
	$r->{'written'} = $r->{$writtenOid};

	print "\tdescription: $r->{'descr'}\n" if $debug;

	my $template = "--template=reads:writes:read:written";
	my $values = "N:$r->{'reads'}:$r->{'writes'}:$r->{'read'}:$r->{'written'}";

	print "\trrdtool update $file\n\t$template $values\n" if $debug;

	RRDs::update ($file, $template, $values);

	return( check_rrds_error() );
};

sub UpdateNetworkAllSNMP($$$$$)
{
	my ($host, $session, $type, $mon, $conf) = @_;
	my ($num);

	my $debug   = $conf->{'debug'};
	my $datadir = $conf->{'datadir'};
	my $ifsOid  = $conf->{'net_ifsOid'} || ".1.3.6.1.2.1.2.1.0";   # interfaces.ifNumber.0
	my $IntOid  = $conf->{'net_IntOid'} || ".1.3.6.1.2.1.2.2.1.2."; # ifDescr
	my $r       = $session->get_request( -varbindlist=>[ $ifsOid ] );

	unless ( defined($r) ) {
		_formatted(undef, "UpdateNetworkAllSNMP: polling net interfaces", "FAILED");
		return 0 
	};

	#  Figure out how many network interfaces we have
	my $max = $r->{$ifsOid};
	_formatted(undef, "UpdateNetworkAllSNMP: looking for network interfaces", $max) if $debug;

	my $eachIntOid;
	my $i = 1; 
	do
	{
		$eachIntOid = $IntOid . $i;
		$r = $session->get_request( -varbindlist=>[ $eachIntOid ] );

		my $descOid   = ".1.3.6.1.2.1.2.2.1.2.$i";   # ifDescr.$i
		my $inOid     = ".1.3.6.1.2.1.2.2.1.10.$i";  # ifInOctets.$i
		my $outOid    = ".1.3.6.1.2.1.2.2.1.16.$i";  # ifOutOctets.$i

		$r = $session->get_request( -varbindlist => [$descOid, $inOid, $outOid] );

		unless ( defined($r) ) 
		{
			printf("\tERROR: %s.\n", $session->error); 
			return 0;
		} 

		_formatted(undef, "        SNMP poll for $r->{$descOid}", "ok") if $debug;

		my $descr = $r->{$descOid};
		my $file  = "$datadir/$host/net_$descr.rrd";
		my $template = "--template=in:out";
		my $values   = "N:$r->{$inOid}:$r->{$outOid}";

		unless ( CheckRRD($file, $type, $mon, $descr, $conf) ) {
			print "ERROR: couldn't find $file\n";
			next;
		};

		print "\trrdtool update $file\n\t$template $values\n" if $debug;

		RRDs::update ($file, $template, $values);
		$i++;
	} 
	until ( $i > $max || $r->{$eachIntOid} eq $mon);

	return( check_rrds_error() );
};

sub UpdateNetworkSNMP($$$$$$)
{
	my ($host, $session, $type, $mon, $descr, $conf) = @_;
	my ($num);

	if ($mon eq "all" ) {
		return UpdateNetworkAllSNMP($host, $session, $type, $mon, $conf);
	};

	my $debug   = $conf->{'debug'};
	my $datadir = $conf->{'datadir'};

	my $file  = "$datadir/$host/$descr.rrd";
	return 0 unless CheckRRD($file, $type, $mon, $descr, $conf);

	my $ifsOid = $conf->{'net_ifsOid'} || ".1.3.6.1.2.1.2.1.0";   # interfaces.ifNumber.0
	my $IntOid = $conf->{'net_IntOid'} || ".1.3.6.1.2.1.2.2.1.2."; # ifDescr.$i
	my $r      = $session->get_request( -varbindlist=>[ $ifsOid ] );

	unless ( defined($r) ) {
		_formatted(undef, "UpdateNetworkSNMP: polling net interfaces", "FAILED");
		return 0 
	};

	#  Figure out how many network interfaces we have
	my $max = $r->{$ifsOid};
	_formatted(undef, "UpdateNetworkSNMP: looking for network interfaces", $max) if $debug;

	my $eachIntOid;
	my $i = 1; 
	do
	{
		$eachIntOid = $IntOid . $i;
		$r = $session->get_request( -varbindlist=>[ $eachIntOid ] );

		$num = $i if ($r->{$eachIntOid} eq $mon);
		$i++;
	} 
	until ( $i > $max || $r->{$eachIntOid} eq $mon);

	unless ($num) 
	{
		_formatted(undef, "        searching for $mon", "FAILED");
		return 0;
	};

	_formatted(undef, "        searching for $mon", "ok") if $debug;

	my $descOid   = ".1.3.6.1.2.1.2.2.1.2.$num";   # ifDescr.$num
	my $inOid     = ".1.3.6.1.2.1.2.2.1.10.$num";  # ifInOctets.$num
	my $outOid    = ".1.3.6.1.2.1.2.2.1.16.$num";  # ifOutOctets.$num

	$r = $session->get_request( -varbindlist => [$descOid, $inOid, $outOid] );

	unless ( defined($r) ) 
	{
		printf("\tERROR: %s.\n", $session->error); 
		return 0;
	} 

	_formatted(undef, "        SNMP poll for $r->{$descOid}", "ok") if $debug;

	my $template = "--template=in:out";
	my $values   = "N:$r->{$inOid}:$r->{$outOid}";

	print "\trrdtool update $file\n\t$template $values\n" if $debug;

	RRDs::update ($file, $template, $values);
	return( check_rrds_error() );
};

sub UpdateSystemSNMP($$$)
{
	my ($file, $session, $conf) = @_;

	my $debug = $conf->{'debug'};

	my $sysUpTime = $conf->{'sys_UpTime'};
	my $admin     = $conf->{'sys_admin'};
	my $hostname  = $conf->{'sys_hostname'};
	my $location  = $conf->{'sys_location'};

	# my $sysUpTime = ".1.3.6.1.2.1.1.3.0";
	# my $admin     = ".1.3.6.1.2.1.1.4.0";
	# my $hostname  = ".1.3.6.1.2.1.1.5.0";
	# my $location  = ".1.3.6.1.2.1.1.6.0";

	my $r = $session->get_request( -varbindlist => [$sysUpTime, $admin, $hostname, $location] );

	unless ( defined($r) )
	{
		printf("UpdateSystemSNMP: ERROR: %s.\n", $session->error); 
		return 0;
	} 
	else 
	{
		print "UpdateSystemSNMP: $r->{'hostname'}  uptime: $r->{'sysUpTime'}  \n" if $debug;

		RRDs::update ( $file, "--template=uptime:admin:host:location", 
			"N:$r->{$sysUpTime}:$r->{$admin}:$r->{$hostname}:$r->{$location}"  );
		return( check_rrds_error() );
	};
};

sub UpdateWebHitsSNMP($$$$$$)
{
	my ($file, $session, $type, $mon, $descr, $conf) = @_;

	my $debug  = $conf->{'debug'};
	my $webOid = $conf->{'webOid'} || ".1.3.6.1.4.1.2021.8.1.101.1";

	#$session->max_msg_size([65535]);
	my $r = $session->get_request( -varbindlist => [$webOid] );

	unless ( defined($r) )
	{
		printf("UpdateWebHitsSNMP: SNMP ERROR: %s.\n", $session->error); 
		return 0;
	};

	my $return = $r->{$webOid};
	my @list = split(/:/, $return);
#	print "result: $return\n" if $debug;
	my $num = 0;

	while ( $list[1] && $list[1] ne "")
	{
	   	my $host  = shift @list;
		my $count = shift @list;

		$r->{$host} = $count; 
		#print "vhost: $host got $count hits\n" if $debug;
		$num++;
	}; 

	if ($num lt 1) 
	{
		_formatted(undef, "        no vhosts!", "FAILED") if $debug;
		return 0;
	};

	print "\tUpdateWebHits: $num vhosts.\n" if $debug;

	unless ($r->{$descr} ) 
	{
		print "\tNo hits for $descr.\n" if $debug;
	};

	print "\trrdtool update $file,\n\t--template=hits\t N:$r->{$descr}\n" if $debug;

	RRDs::update ( $file, "--template=hits", "N:$r->{$descr}"  );
	return( check_rrds_error() );
};

sub UpdateMailSNMP($$$$$$)
{
	my ($file, $session, $type, $mon, $descr, $conf) = @_;
	my ($r);

	my $debug = $conf->{'debug'};

	my $send = $conf->{'mail_send'} || ".1.3.6.1.4.1.2021.8.1.101.2";  #  send      Oid
	my $smtp = $conf->{'mail_smtp'} || ".1.3.6.1.4.1.2021.8.1.101.3";  #  smtp      Oid
	my $pop3 = $conf->{'mail_pop3'} || ".1.3.6.1.4.1.2021.8.1.101.4";  #  pop3      Oid
	my $imap = $conf->{'mail_imap'} || ".1.3.6.1.4.1.2021.8.1.101.5";  #  imap      Oid
	my $rbl  = $conf->{'mail_rbl'}  || ".1.3.6.1.4.1.2021.8.1.101.6";  #  rbl       Oid
	my $web  = $conf->{'mail_web'}  || ".1.3.6.1.4.1.2021.8.1.101.7";  #  webmail   Oid
	my $spam = $conf->{'mail_spam'} || ".1.3.6.1.4.1.2021.8.1.101.8";  #  spamassassin Oid
	my $qs   = $conf->{'mail_qs'}   || ".1.3.6.1.4.1.2021.8.1.101.9";  #  qmailscanner Oid

	if    ( $mon eq "mua" ) 
	{
		$r = $session->get_request( -varbindlist => [$pop3, $imap, $smtp, $web] );
	}
	elsif ( $mon eq "send" )
	{
		$r = $session->get_request( -varbindlist => [$send] );
	}
	elsif ( $mon eq "rbl" || $mon eq "malformed")
	{
		$r = $session->get_request( -varbindlist => [$rbl] );
	}	
	elsif ( $mon eq "spam")
	{
		$r = $session->get_request( -varbindlist => [$spam, $qs] );
	}

	unless ( defined($r) ) 
	{
		printf("UpdateMailSNMP: ERROR: %s.\n", $session->error); 
		return 0;
	} 

	_formatted(undef, "UpdateMailSNMP: polling for $mon", "ok") if $debug;

	$r = CramKeyValuePairsIntoHash($r);

	print "\tupdating $descr\n" if $debug;

#	foreach my $key (sort keys %$r ) 
#	{
#		print "$key: $r->{$key}\t" if $debug;
#	};
#	print "\n" if $debug;

	my ($template, $values);

	if ( $mon eq "smtp" || $mon eq "rbl" )
	{
		#spamhaus spamcop dsbl ordb maps sorbs rfc-ignorant services njabl monkeys visi
		#other badhelo badmailfrom badmailto dns chkuser spf virus

		$template = "--template=smtp1:smtp2:smtp3:smtp4:smtp5:smtp6:smtp7:smtp8:smtp9:smtp10:smtp11:smtp12:smtp13:smtp14:smtp15:smtp16:smtp17:smtp18:smtp19:smtp20";
		$values   = "N:$r->{'spamhaus'}:$r->{'spamcop'}:$r->{'dsbl'}:$r->{'ordb'}:$r->{'maps'}:$r->{'sorbs'}:$r->{'ignorant'}:$r->{'services'}:1:1:1:1:$r->{'badhelo'}:$r->{'badmailfrom'}:$r->{'badmailto'}:$r->{'dns'}:$r->{'other'}:$r->{'chkuser'}:$r->{'spf'}:$r->{'virus'}";
	}
	elsif ( $mon eq "send" )
	{
		$template = "--template=delivery_success:delivery_deferral:delivery_failure:message_bytes:message_new:message_bounce:message_end:concurrencyremote";
		$values    = "N:$r->{'delivery_success'}:$r->{'delivery_deferral'}:$r->{'delivery_failure'}:$r->{'message_bytes'}:$r->{'message_new'}:$r->{'message_bounce'}:$r->{'message_end'}:$r->{'concurrencyremote'}";
	}
	elsif ( $mon eq "mua" )
	{
		$template = "--template=pop3_connect:pop3_ssl_connect:pop3_success:pop3_ssl_success:imap_connect:imap_ssl_connect:imap_success:imap_ssl_success:webmail_connect:webmail_success:smtpauth_connect:smtpauth_success";
		$values = "N:$r->{'pop3_connect'}:$r->{'pop3_ssl_connect'}:$r->{'pop3_success'}:$r->{'pop3_ssl_success'}:$r->{'imap_connect'}:$r->{'imap_ssl_connect'}:$r->{'imap_success'}:$r->{'imap_ssl_success'}:$r->{'webmail_connect'}:$r->{'webmail_success'}:$r->{'smtp_auth_connect'}:$r->{'smtp_auth_success'}";
	}
	elsif ( $mon eq "spam" )
	{
		$template = "--template=sa1:sa2:sa3:sa4:sa5:sa6:sa7:sa8:sa9:sa10";
	 	$values   = "N:$r->{'sa_spam'}:$r->{'sa_ham'}:$r->{'spam_score'}:$r->{'ham_score'}:$r->{'threshhold'}:$r->{'spam_seconds'}:$r->{'ham_seconds'}:$r->{'ham_bytes'}:$r->{'spam_bytes'}:U";
	};

	print "\trrdtool update $file\n\t$template\n\t$values\n" if $debug;

	RRDs::update ( $file, $template, $values );

	return( check_rrds_error() );
};

sub UpdateMysqlSNMP($$$$$$)
{
	my ($file, $session, $type, $mon, $descr, $conf) = @_;
	my ($r);

	my $debug      = $conf->{'debug'};
	my $queries    = $conf->{'mysql_queries'};
	my $type_query = $conf->{'mysql_type'};

	if    ( $mon eq "queries" ) 
	{
		$r = $session->get_request( -varbindlist => [$queries] );
	}
	elsif ( $mon eq "type_queries" )
	{
		$r = $session->get_request( -varbindlist => [$type_query] );
	}
	elsif ( $mon eq "combined" )
	{
		$r = $session->get_request( -varbindlist => [$type_query] );
	}

	unless ( defined($r) ) 
	{
		printf("UpdateMysqlSNMP: ERROR: %s.\n", $session->error); 
		return 0;
	} 

	_formatted(undef, "UpdateMysqlSNMP: polling mysql stats", "ok") if $debug;

	$r = CramKeyValuePairsIntoHash($r);

	print "UpdateMysqlSNMP: \n" if $debug;

	my ($template, $values);

	if ( $mon eq "queries" )
	{
		$template = "--template=mysql_queriessec:mysql_connections:mysql_slowqueries", 
		$values   = "N:$r->{'mysql_queriessec'}:$r->{'mysql_connections'}:$r->{'mysql_slowqueries'}"
	}
	elsif ( $mon eq "type_queries" )
	{
		$template = "--template=mysql_select:mysql_changedb:mysql_update:mysql_insert:mysql_delete", 
		$values   = "N:$r->{'mysql_select'}:$r->{'mysql_changedb'}:$r->{'mysql_update'}:$r->{'mysql_insert'}:$r->{'mysql_delete'}"
	}
	elsif ( $mon eq "combined" )
	{
		$template = "--template=my1:my2:my3:my4:my5:my6:my7:my8", 
		$values   = "N:$r->{'select'}:$r->{'changedb'}:$r->{'update'}:$r->{'insert'}:$r->{'delete'}:$r->{'connections'}:$r->{'slow'}:$r->{'queries'}"
	};

	print "\trrdtool update $file\n\t$template\n\t$values\n" if $debug;

	RRDs::update ( $file, $template, $values );

	return( check_rrds_error() );
};

sub UpdateMbmonSNMP($$$$$$)
{
	my ($file, $session, $type, $mon, $descr, $conf) = @_;
	my ($r);

	my $debug       = $conf->{'debug'};
	my $temperature = $conf->{'temperature'};
	my $voltage     = $conf->{'voltage'};
	my $fanspeed    = $conf->{'fanspeed'};

	if    ( $mon eq "temperature" ) 
	{
		$r = $session->get_request( -varbindlist => [$temperature] );
	}
	elsif ( $mon eq "voltage" )
	{
		$r = $session->get_request( -varbindlist => [$voltage] );
	}
	elsif ( $mon eq "fanspeed" )
	{
		$r = $session->get_request( -varbindlist => [$fanspeed] );
	}

	unless ( defined($r) ) 
	{
		printf("UpdateMbmonSNMP: ERROR: %s.\n", $session->error); 
		return 0;
	} 
	else
	{
		$r = CramKeyValuePairsIntoHash($r);

		print "UpdateMbmonSNMP: \n" if $debug;

		print "\tupdate $file, $descr \n" if $debug;
		foreach my $key (sort keys %$r ) 
		{
			print "$key: $r->{$key}\t" if $debug;
		};
		print "\n" if $debug;

		if ( $mon eq "temperature" )
		{
			RRDs::update ( $file, 
				"--template=mbmon_temp0:mbmon_temp1:mbmon_temp2", 
				"N:$r->{'mbmon_temp0'}:$r->{'mbmon_temp1'}:$r->{'mbmon_temp2'}"
			);
		}
		elsif ( $mon eq "voltage" )
		{
			RRDs::update ( $file, 
				"--template=mbmon_vc0:mbmon_vc1:mbmon_v33:mbmon_v50p:mbmon_v12p:mbmon_v12n:mbmon_v50n", 
				"N:$r->{'mbmon_vc0'}:$r->{'mbmon_vc1'}:$r->{'mbmon_v33'}:$r->{'mbmon_v50p'}:$r->{'mbmon_v12p'}:$r->{'mbmon_v12n'}:$r->{'mbmon_v50ni'}"	);
		}	

		elsif ( $mon eq "fanspeed" )
		{
			RRDs::update ( $file, 
				"--template=mbmon_fan0:mbmon_fan1:mbmon_fan2",
				"N:$r->{'mbmon_fan0'}:$r->{'mbmon_fan1'}:$r->{'mbmon_fan2'}"	);
		}
	
		return( check_rrds_error() );
	};
};

sub UpdateApacheSNMP($$$$$$)
{
	my ($file, $session, $type, $mon, $descr, $conf) = @_;
	my ($r);

	my $debug   = $conf->{'debug'};
	my $apache 	= $conf->{'apacheOid'};
	
	$r = $session->get_request( -varbindlist => [$apache] );
	
	unless ( defined($r) ) 
	{
		printf("UpdateApacheSNMP: ERROR: %s.\n", $session->error); 
		return 0;
	} 

	_formatted(undef, "UpdateApacheSNMP: polling apache stats", "ok") if $debug;

	$r = CramKeyValuePairsIntoHash($r);

	my ($template, $values);

	if ( $mon eq "workers")
	{
		$template = "--template=apache_workers_busy:apache_workers_idle";
		$values   = "N:$r->{'apache_busy_workers'}:$r->{'apache_idle_workers'}";
	}
	elsif( $mon eq "bytes_per_req")
	{
		$template = "--template=apache_per_req";
		$values   = "N:$r->{'bytes_per_req'}";
	}
	elsif( $mon eq "req_per_sec")
	{
		$template = "--template=apache_req_per_sec";
		$values   = "N:$r->{'req_per_sec'}";
	}
	elsif( $mon eq "combined")
	{
		$template = "--template=workers_busy:workers_idle:bytes_per_req:req_per_sec";
		$values   = "N:$r->{'apache_busy_workers'}:$r->{'apache_idle_workers'}:$r->{'bytes_per_req'}:$r->{'req_per_sec'}";
	}

	print "\trrdtool update $file\n\t$template\n\t$values\n" if $debug;

	RRDs::update ( $file, $template, $values);

	return( check_rrds_error() );
}

sub UpdatePFSNMP($$$$$$)
{
	my ($file, $session, $type, $mon, $descr, $conf) = @_;
	my ($r);

	my $debug     = $conf->{'debug'};
	my $states_entries  = $conf->{'states_entries'};
	my $states_searches = $conf->{'states_searches'};
	my $states_inserts  = $conf->{'states_inserts'};
	my $states_removals = $conf->{'states_removals'};

    my $bytes_v4_in  = $conf->{'bytes_v4_in'};
	my $bytes_v4_out = $conf->{'bytes_v4_out'};
	my $bytes_v6_in  = $conf->{'bytes_v6_in'};
	my $bytes_v6_out = $conf->{'bytes_v6_out'};
    my $passed     = $conf->{'passed'}; 
	my $blocked    = $conf->{'blocked'};
	
	if    ( $mon eq "states" ) 
	{
		$r = $session->get_request(  -varbindlist => [$states_entries, $states_searches, $states_inserts, $states_removals] );
	}
	elsif ( $mon eq "ipbytes" )
	{
		$r = $session->get_request( -varbindlist => [$bytes_v4_in, $bytes_v4_out, $bytes_v6_in, $bytes_v6_out] );
	}
	elsif ( $mon eq "firewall" )
	{
		$r = $session->get_request( -varbindlist => [$passed, $blocked] );
	}

	unless ( defined($r) ) 
	{
		printf("UpdatePFSNMP: ERROR: %s.\n", $session->error); 
		return 0;
	} 
	else
	{

		print "UpdatePFSNMP: \n" if $debug;

		print "\tupdate $file, $descr \n" if $debug;
        foreach my $key (sort keys %$r ) 
		{
            print "$key  : $r->{$key}\t" if $debug;
		};

        print "\n" if $debug;


		if ( $mon eq "states")
		{
            RRDs::update ( $file, 
				"--template=states_entries:states_searches:states_inserts:states_removals", 
				"N:$r->{$states_entries}:$r->{$states_searches}:$r->{$states_inserts}:$r->{$states_removals}"	);
			
		}
		elsif( $mon eq "ipbytes")
		{
			RRDs::update ( $file, 
				"--template=bytes_v4_in:bytes_v4_out:bytes_v6_in:bytes_v6_out", 
				"N:$r->{$bytes_v4_in}:$r->{$bytes_v4_out}:$r->{$bytes_v6_in}:$r->{$bytes_v6_out}"	);
		
		}
		elsif( $mon eq "firewall")
		{
			RRDs::update ( $file, 
				"--template=passed:blocked", 
				"N:$r->{$passed}:$r->{$blocked}"	);
		}
	}
};


sub _formatted
{
	my ($self, $mess, $result) = @_;

	my $dots;
	my $len = length($mess);
	if ($len < 65) { until ( $len == 65 ) { $dots .= "."; $len++ }; };
	print "$mess $dots $result\n";
}

1;
__END__

Copyright (c) 2003-2004, The Network People, Inc. All Rights Reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

Neither the name of the The Network People, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
